// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:meta/meta.dart';
import 'package:sembast/sembast.dart';
import 'package:sembast/sembast_io.dart';

/// Stores all data (rooms, users, events) somewhere.
abstract class Store {
  bool get isOpen;

  Future<void> open();
  Future<void> close();

  Future<String> getAccessToken();
  Future<void> setAccessToken(String value);

  Future<String> getSyncToken();
  Future<void> setSyncToken(String value);
}

class SembastStore implements Store {

  var _isOpen = false;
  bool get isOpen => _isOpen;

  final String _path;
  SembastStore({@required String path}) : _path = path;

  Database _db;

  Future<void> open() async {
    _db = await databaseFactoryIo.openDatabase(_path);
    _isOpen = true;
  }
  Future<void> close() async {
    await _db.close();
    _isOpen = false;
  }

  static const accessTokenKey = 'access_token';
  Future<String> getAccessToken() async => await _db.get(accessTokenKey);
  Future<void> setAccessToken(String value) async {
    await _db.put(value, accessTokenKey);
  }

  static const syncTokenKey = 'sync_token';
  Future<String> getSyncToken() async => await _db.get(syncTokenKey);
  Future<void> setSyncToken(String value) async {
    await _db.put(value, syncTokenKey);
  }
}